package simulator.elevatormodules;

import jSimPack.SimTime;
import simulator.framework.TimeSensitive;
import simulator.framework.Timer;
import simulator.payloads.*;

/**
 * Models activities related to coins.
 *
 * @author justinr2
 */
public class ChangeMachine extends Module implements TimeSensitive {

    
    private CoinInPayload coinInState;  //send this state
    private CoinTrayPayload coinTrayState; //send this state
    private CoinOutPayload coinOutState; //receive this state
    private int internalCoinCount = 0;
    private boolean isCoinOut = false;
    public final static SimTime COIN_IN_PULSE_WIDTH = new SimTime(210, SimTime.SimTimeUnit.MILLISECOND);
    private Timer timer;


    ChangeMachine(SimTime period, boolean verbose) {
        super(period, "ChangeMachine", verbose);

        coinInState = new CoinInPayload();
        coinInState.value = false;
        physicalConnection.sendTimeTriggered(coinInState, period);

        coinTrayState = new CoinTrayPayload();
        coinTrayState.coinCount=0;
        physicalConnection.sendTimeTriggered(coinTrayState, period);

        coinOutState = new CoinOutPayload();
        physicalConnection.registerEventTriggered(coinOutState);
        physicalConnection.registerTimeTriggered(coinOutState);

        timer = new Timer(this);

        //previousSpeed = driveSpeedState.speed;
        //previousDir = driveSpeedState.direction;
    }

    @Override
    public void receive(CoinOutPayload msg) {
        if (coinOutState.value && !isCoinOut) {
            log ("Releasing coin to tray");
            //first time to receive this value, so return a coin
            isCoinOut = true;
            if (internalCoinCount == 0) {
                throw new RuntimeException("Error, tried to return a coin but no coins have been received.");
            }
            internalCoinCount--;
            coinTrayState.coinCount++;
            physicalConnection.sendOnce(coinTrayState);
        } else if (!coinOutState.value && isCoinOut) {
            //end of pulse
            log ("End of CoinOut pulse");
            isCoinOut = false;
        }
    }

    //passenger interface methods
    public void insertCoin() {
        log ("Insert a coin");
        internalCoinCount++;
        coinInState.value = true;
        physicalConnection.sendOnce(coinInState);
        timer.start(COIN_IN_PULSE_WIDTH);  //start a timer to lower the pulse
    }

    //timer callback occurs when it's time to make CoinIn false
    public void timerExpired(Object callbackData) {
        log ("End of CoinIn pulse");
        coinInState.value = false;
        physicalConnection.sendOnce(coinInState);
    }

    public int getCoinsFromTray() {
        int coinTrayCount = coinTrayState.coinCount;
        log ("Retrieved " + coinTrayCount + " coins from cointray.");
        coinTrayState.coinCount = 0;
        physicalConnection.sendOnce(coinTrayState);
        return coinTrayCount;
    }


    @Override
    public String toString() {
        return "ChangeMachine";
    }


}
