package simulator.elevatormodules;

import jSimPack.SimTime;
import simulator.elevatormodules.passenger.PassengerControl;
import simulator.elevatormodules.passenger.PassengerEventSender;
import simulator.framework.CANNetwork.CanConnection;
import simulator.framework.Harness;
import simulator.framework.NetworkScheduler.Connection;
import simulator.framework.Networkable;

/**
 * @author Justin Ray
 */
public abstract class Module extends Networkable implements PassengerEventSender {

    final protected CanConnection canNetworkConnection;
    final protected Connection physicalConnection;
    protected PassengerControl passengerControl;
    protected SimTime period;
    protected final String name;
    private final boolean verbose;

    public Module(SimTime period, String name, boolean verbose) {
        this.period = period;
        this.name = name;
        canNetworkConnection = Harness.getCANNetwork().getCanConnection();
        physicalConnection = Harness.getPhysicalNetwork().getFrameworkConnection(this);
        this.verbose = verbose;
    }

    protected void log(Object... msg) {
        if(verbose) Harness.log(name, msg);
    }

    @Override
    public String toString() {
        return name;
    }

    /**
     * Keep a reference to passengerControl so modules can send passenger events.
     * @param pc - current passenger control object
     */
    public void registerPassengerControl(PassengerControl pc) {
        passengerControl = pc;
    }


}