package simulator.framework;

import jSimPack.SimTime;
import java.util.*;
import simulator.payloads.CanMailbox;

public class CANNetwork extends NetworkScheduler {
   
    public class CanConnection {

        private NetworkConnection conn;

        public CanConnection() {
            conn = new Connection(null);
        }

        public void registerTimeTriggered(CanMailbox writeback) {
            conn.registerTimeTriggered(writeback);
        }
        
        public void sendTimeTriggered(CanMailbox msg, SimTime period) {
            registerCanSender(msg);
            conn.sendTimeTriggered(msg, period);
        }

        public void setEnabled(boolean enabled) {
            conn.setEnabled(enabled);
        }
    }

    CANNetwork(SimTime bitWidth) {
        super(bitWidth);
    }


    private Set<Integer> canSenders = new HashSet<Integer>();

    public CanConnection getCanConnection() {
        return new CanConnection();
    }

    public Connection getFrameworkConnection()
    {
        return getFrameworkConnection(null);
    }

    public Connection getFrameworkConnection(Networkable networkNode)
    {
        return new Connection(networkNode);
    }


    /**
     * Informs the network that the specified message is the only one that
     * will have this message ID.  If another message with the same ID is
     * sent on the network, a runtime exception will be thrown.
     */
    private void registerCanSender(CanMailbox p) {
        if (canSenders.contains(p.getMessageId())) {
            throw new RuntimeException("a CAN message with ID " + p.getMessageId() + " is already being sent.");
        }
        canSenders.add(p.getMessageId());
    }

    @Override
        public String toString() {
            return "CANNetwork";
        }
}
