package simulator.framework;

/**
 * Receives <code>HarnessEvent</code>s that are broadcast by the
 * <code>Harness</code>.  Implementing classes register to receive events by
 * calling <code>Harness.addListener(this,type)</code>, where
 * <code>type</code> is the desired event type.
 *
 * @author Christopher Martin
 * @author Kenny Stauffer
 */

public interface HarnessEventListener
{
    /**
     * Invoked when the <code>Harness</code> broadcasts an event to the
     * listeners that have called {@link
     * Harness#addListener(HarnessEventListener,int)} to receive events of
     * this type.  The event <code>e</code> will be of a type this
     * <code>HarnessEventListener</code> registered for.
     */
    public void eventDispatched(HarnessEvent e);
}
