/**
 * @brief
 * Displays the status of the elevator, using Java Swing.
 *
 * @author
 * Kenny Stauffer (kstauffer@cmu.edu)
 */
package simulator.framework;

import jSimPack.BreakpointListener;
import jSimPack.SimTime;
import simulator.payloads.*;
import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class SwingDisplay extends javax.swing.JFrame implements BreakpointListener {

    private static final long serialVersionUID = 0;
    private final Collection<Widget> widgets;
    public final NetworkConnection cni;
    public final NetworkConnection fi;
    private final boolean verbose;
    //TODO define payloads
    //TODO define gui objects
//    private final JSlider positionSlider;
//    private final JTextField timeField;
//    private final JTextField posField;
//    private final JSlider speedSlider;
//    private final JTextField speedField;
//    private final JTextField weightField;
//    private final JTextField indField;
//    private final JCheckBox upLanternCheckBox;
//    private final JCheckBox downLanternCheckBox;
//    private final JTextField desDirTextField;
    private final JSpinner realtimeField;
    //number of nanoseconds between GUI refreshes
    private final static long REFRESH_INTERVAL_MS = 50;
    private java.util.Timer threadTimer = new java.util.Timer();

    private static interface Widget {

        public void update();
    }

    private class ChutePanel extends JPanel implements Widget {

        private static final long serialVersionUID = 0;
        final JSlider positionSlider;
        final VendCarriagePositionPayload positionState;

        public ChutePanel() {
            super();

            //register objects
            positionState = new VendCarriagePositionPayload();
            fi.registerTimeTriggered(positionState);

            setLayout(new GridBagLayout());
            GridBagConstraints cons = new GridBagConstraints();

            //add labels

            JLabel chuteLabel = new JLabel();
            chuteLabel.setText("Chute Counts:");
            //chuteLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            cons.gridx = 0;
            cons.gridy = 0;
            cons.anchor = GridBagConstraints.EAST;
            cons.ipadx = 5;
            cons.ipady = 5;
            //cons.fill = GridBagConstraints.HORIZONTAL;
            add(chuteLabel, cons);

            JLabel positionSensorLabel = new JLabel();
            positionSensorLabel.setText("VendPosition:");
            //positionSensorLabel.setHorizontalAlignment(SwingConstants.CENTER);
            cons = new GridBagConstraints();
            cons.gridx = 0;
            cons.gridy = 1;
            cons.anchor = GridBagConstraints.EAST;
            cons.ipadx = 5;
            cons.ipady = 5;
            add(positionSensorLabel, cons);

            JLabel vendCarriageLabel = new JLabel();
            vendCarriageLabel.setText("Vend Carriage Position:");
            //vendCarriageLabel.setHorizontalAlignment(SwingConstants.CENTER);
            cons = new GridBagConstraints();
            cons.gridx = 0;
            cons.gridy = 2;
            cons.anchor = GridBagConstraints.EAST;
            cons.ipadx = 5;
            cons.ipady = 5;
            add(vendCarriageLabel, cons);

            JLabel sodaButtonLabel = new JLabel();
            sodaButtonLabel.setText("SodaButtons:");
            //sodaButtonLabel.setHorizontalAlignment(SwingConstants.CENTER);
            cons = new GridBagConstraints();
            cons.gridx = 0;
            cons.gridy = 3;
            cons.anchor = GridBagConstraints.EAST;
            cons.ipadx = 5;
            cons.ipady = 5;
            add(sodaButtonLabel, cons);

            //add slider for the vend carriage

            /* position slider */
            positionSlider = new JSlider();
            positionSlider.setFocusable(false);
            positionSlider.setMaximum(700);
            cons = new GridBagConstraints();
            cons.gridy = 2;
            cons.gridx = 1;
            cons.gridwidth = GridBagConstraints.REMAINDER;
            cons.fill = GridBagConstraints.HORIZONTAL;
            cons.ipady = 5;
            add(positionSlider, cons);

            for (int x=1; x < SodaMachine.NUM_CHUTES+1; x++ ) {
                int index = x;
                JComponent jc;

                //add sodaChute
                jc= new ChuteCountLabel(index);
                widgets.add((Widget) jc);
                cons = new GridBagConstraints();
                cons.weighty = 1.0;
                cons.weightx = 0.2;
                cons.gridy = 0;
                cons.gridx = x;
                add(jc, cons);

                //add position sensor
                jc= new PositionSensorLabel(index);
                widgets.add((Widget) jc);
                cons = new GridBagConstraints();
                cons.weighty = 1.0;
                cons.weightx = 0.2;
                cons.gridy = 1;
                cons.gridx = x;
                add(jc, cons);

                jc= new ButtonLightCheckbox(index);
                widgets.add((Widget) jc);
                cons = new GridBagConstraints();
                cons.weighty = 1.0;
                cons.weightx = 0.2;
                cons.gridy = 3;
                cons.gridx = x;
                add(jc, cons);
            }

            widgets.add(this);
        }

        public void update() {
            positionSlider.setValue((int)positionState.position);
        }

        private class ButtonLightCheckbox extends JCheckBox implements Widget {

            private static final long serialVersionUID = 0;
            final ButtonPayload call;
            final LightPayload light;

            public ButtonLightCheckbox(int index) {
                super();
                setFocusable(false);
                call = new ButtonPayload(index);
                //cni.registerTimeTriggered(call);
                fi.registerTimeTriggered(call);
                light = new LightPayload(index);
                fi.registerTimeTriggered(light);
            }

            public void update() {
                setSelected(call.pressed);
                //setBackground(light.lighted ? Color.GREEN : Color.getHSBColor(0.0f, 0.0f, 0.93f));
                setBackground(light.lighted ? Color.GREEN : Color.BLACK);
            }
        }

        private class PositionSensorLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final VendPositionPayload position;

            public PositionSensorLabel(int index) {
                super("F");
                position = new VendPositionPayload(index);
                fi.registerTimeTriggered(position);
            }

            public void update() {
                setText(position.value ? "T" : "F");
                setBackground(position.value ? Color.GREEN : Color.RED);
            }
        }

        private class ChuteCountLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final SodaChutePayload sodaChute;

            public ChuteCountLabel(int index) {
                super("50");
                sodaChute = new SodaChutePayload(index);
                fi.registerTimeTriggered(sodaChute);
            }

            public void update() {
                setText(Integer.toString(sodaChute.sodaCount));
            }
        }


    }

    private class SodaPanel extends JPanel{

        private static final long serialVersionUID = 0;
        final ChutePanel chutePanel;
        final ReturnCheckBox returnCheckBox;
        final CoinInLabel coinInLabel;
        final CoinOutLabel coinOutLabel;
        final MotorDirectionLabel motorDirectionLabel;
        final SodaTrayLabel sodaTrayLabel;
        final CoinTrayLabel coinTrayLabel;

        public SodaPanel() {
            super();

            //gui stuff
            setLayout(new GridBagLayout());
            GridBagConstraints cons;

            chutePanel = new ChutePanel();
            cons = new GridBagConstraints();
            cons = new GridBagConstraints();
            cons.gridy = 0;
            cons.gridx = 0;
            cons.gridwidth = GridBagConstraints.REMAINDER;
            cons.fill = GridBagConstraints.HORIZONTAL;
            add(chutePanel, cons);

            JLabel l;
            l = new JLabel("CoinReturn:");
            cons = new GridBagConstraints();
            cons.gridy = 1;
            cons.gridx = 0;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.EAST;
            add(l, cons);

            returnCheckBox = new ReturnCheckBox();
            widgets.add(returnCheckBox);
            cons = new GridBagConstraints();
            cons.gridy = 1;
            cons.gridx = 1;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.WEST;
            add(returnCheckBox, cons);

            l = new JLabel("CoinIn:");
            cons = new GridBagConstraints();
            l.setHorizontalAlignment(SwingConstants.CENTER);
            cons.gridy = 2;
            cons.gridx = 0;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.EAST;
            add(l, cons);

            coinInLabel = new CoinInLabel();
            widgets.add(coinInLabel);
            cons = new GridBagConstraints();
            cons.gridy = 2;
            cons.gridx = 1;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.WEST;
            add(coinInLabel, cons);

            l = new JLabel("CoinOut:");
            cons = new GridBagConstraints();
            l.setHorizontalAlignment(SwingConstants.CENTER);
            cons.gridy = 3;
            cons.gridx = 0;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.EAST;
            add(l, cons);


            coinOutLabel = new CoinOutLabel();
            widgets.add(coinOutLabel);
            cons = new GridBagConstraints();
            cons.gridy = 3;
            cons.gridx = 1;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.WEST;
            add(coinOutLabel, cons);


            l = new JLabel("CoinTray:");
            cons = new GridBagConstraints();
            l.setHorizontalAlignment(SwingConstants.CENTER);
            cons.gridy = 1;
            cons.gridx = 2;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.EAST;
            add(l, cons);

            coinTrayLabel = new CoinTrayLabel();
            widgets.add(coinTrayLabel);
            cons = new GridBagConstraints();
            cons.gridy = 1;
            cons.gridx = 3;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.WEST;
            add(coinTrayLabel, cons);

            l = new JLabel("SodaTray:");
            cons = new GridBagConstraints();
            l.setHorizontalAlignment(SwingConstants.CENTER);
            cons.gridy = 2;
            cons.gridx = 2;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.EAST;
            add(l, cons);

            sodaTrayLabel = new SodaTrayLabel();
            widgets.add(sodaTrayLabel);
            cons = new GridBagConstraints();
            cons.gridy = 2;
            cons.gridx = 3;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.WEST;
            add(sodaTrayLabel, cons);

            l = new JLabel("VendMotor:");
            cons = new GridBagConstraints();
            l.setHorizontalAlignment(SwingConstants.CENTER);
            cons.gridy = 3;
            cons.gridx = 2;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.EAST;
            add(l, cons);

            motorDirectionLabel = new MotorDirectionLabel();
            widgets.add(motorDirectionLabel);
            cons = new GridBagConstraints();
            cons.gridy = 3;
            cons.gridx = 3;
            cons.ipady = 5;
            cons.ipadx = 5;
            cons.anchor = GridBagConstraints.WEST;
            add(motorDirectionLabel, cons);
        }

        private class ReturnCheckBox extends JCheckBox implements Widget {

            private static final long serialVersionUID = 0;
            final CoinReturnButtonPayload call;

            public ReturnCheckBox() {
                super();
                setFocusable(false);
                call = new CoinReturnButtonPayload();
                //cni.registerTimeTriggered(call);
                fi.registerTimeTriggered(call);
            }

            public void update() {
                setSelected(call.pressed);
            }
        }

        private class CoinInLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final CoinInPayload coinIn;

            public CoinInLabel() {
                super("F");
                coinIn = new CoinInPayload();
                fi.registerTimeTriggered(coinIn);
            }

            public void update() {
                setText(coinIn.value ? "T" : "F");
                setBackground(coinIn.value ? Color.GREEN : Color.RED);
            }
        }

        private class CoinOutLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final CoinOutPayload coinOut;

            public CoinOutLabel() {
                super("F");
                coinOut = new CoinOutPayload();
                fi.registerTimeTriggered(coinOut);
            }

            public void update() {
                setText(coinOut.value ? "T" : "F");
                setBackground(coinOut.value ? Color.GREEN : Color.RED);
            }
        }

        private class MotorDirectionLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final VendMotorPayload vendMotor;

            public MotorDirectionLabel() {
                super("S");
                vendMotor = new VendMotorPayload();
                fi.registerTimeTriggered(vendMotor);
            }

            public void update() {
                switch (vendMotor.direction) {
                    case LEFT:
                        setText("L");
                        break;
                    case RIGHT:
                        setText("R");
                        break;
                    case STOP:
                        setText("S");
                        break;
                }
                //setBackground(coinTray.value ? Color.GREEN : Color.RED);
            }
        }

        private class SodaTrayLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final SodaTrayPayload sodaTray;

            public SodaTrayLabel() {
                super("0");
                sodaTray = new SodaTrayPayload();
                fi.registerTimeTriggered(sodaTray);
            }

            public void update() {
                setText(Integer.toString(sodaTray.sodaValue));
                //setBackground(coinTray.value ? Color.GREEN : Color.RED);
            }
        }

        private class CoinTrayLabel extends JLabel implements Widget {

            private static final long serialVersionUID = 0;
            final CoinTrayPayload coinTray;

            public CoinTrayLabel() {
                super("0");
                coinTray = new CoinTrayPayload();
                fi.registerTimeTriggered(coinTray);
            }

            public void update() {
                setText(Integer.toString(coinTray.getCoinCount()));
                //setBackground(coinTray.value ? Color.GREEN : Color.RED);
            }
        }
    }

    /**
     * Creates a new SwingDisplay that repaints its components at a fixed
     * frequency according to the simulation clock.
     */
    public SwingDisplay(boolean verbose) {
        /* This is ugly, because most of it was done with Netbeans.  If you
         * want to mess around with figuring out all the values by hand, be my
         * guest.  --KSS
         */
        super();

        this.verbose = verbose;

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        cni = Harness.getCANNetwork().getFrameworkConnection();
        fi = Harness.getPhysicalNetwork().getFrameworkConnection();
        widgets = new ArrayList<Widget>();

        JPanel widgetPanel = new JPanel();
        widgetPanel.setLayout(new GridBagLayout());
        GridBagConstraints cons;

        JPanel panel = new JPanel();
        Dimension dims = new Dimension(400, 200);
        SodaPanel sp = new SodaPanel();
        sp.setBorder(BorderFactory.createTitledBorder("Soda Machine"));
        sp.setMinimumSize(dims);
        sp.setMaximumSize(dims);
        sp.setPreferredSize(dims);
        cons = new GridBagConstraints();
        cons.gridy = 0;
        cons.gridx = 0;
        cons.fill = GridBagConstraints.VERTICAL;
        widgetPanel.add(sp, cons);



        JLabel label;

        panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        label = new JLabel("Realtime Execution Rate");
        cons = new GridBagConstraints();
        cons.gridy = 0;
        cons.gridx = 0;
        cons.ipadx = 5;
        cons.anchor = GridBagConstraints.WEST;
        panel.add(label, cons);

        realtimeField = new JSpinner(
                new SpinnerNumberModel(Harness.getRealtimeRate(),
                0.0, Double.POSITIVE_INFINITY, 0.1));
        realtimeField.setEditor(new JSpinner.NumberEditor(realtimeField, "0.0"));
        ((JSpinner.NumberEditor) realtimeField.getEditor()).getTextField().setColumns(3);
        realtimeField.setFocusable(true);
        cons = new GridBagConstraints();
        cons.gridy = 0;
        cons.gridx = 1;
        cons.ipadx = 5;
        cons.anchor = GridBagConstraints.WEST;
        panel.add(realtimeField, cons);

        realtimeField.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent ev) {
                JSpinner source = (JSpinner) ev.getSource();
                Harness.setRealtimeRate((Double) source.getValue());
                if (Harness.simulationIsBlocked()) {
                    Harness.stepSimulation();
                }
            }
        });

        JButton pauseButton = new JButton("Pause");
        cons = new GridBagConstraints();
        cons.gridy = 0;
        cons.gridx = 2;
        cons.insets = new Insets(5, 5, 5, 5);
        cons.anchor = GridBagConstraints.WEST;
        pauseButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                realtimeField.setValue(0.0);
            }
        });
        panel.add(pauseButton, cons);

        JButton maxRateButton = new JButton("Fast Forward");
        cons = new GridBagConstraints();
        cons.gridy = 0;
        cons.gridx = 3;
        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.weightx = 1.0;
        cons.anchor = GridBagConstraints.WEST;
        maxRateButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                realtimeField.setValue(Double.POSITIVE_INFINITY);
            }
        });
        panel.add(maxRateButton, cons);

        cons = new GridBagConstraints();
        cons.gridy = 1;
        cons.gridx = 0;
        cons.weightx = 1.0;
        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.anchor = GridBagConstraints.EAST;
        cons.fill = GridBagConstraints.HORIZONTAL;
        widgetPanel.add(panel, cons);

        getContentPane().add(widgetPanel);
        pack();

        Harness.addBreakpointListener(this);

        threadTimer.schedule(new RefreshTimer(), 50, 50);
    }

    private class RefreshTimer extends TimerTask {

        @Override
        public void run() {
            Harness.interleaveLock();
            try {
                updateControls();
            } finally {
                Harness.interleaveUnlock();
            }
        }
    }

    private void updateControls() {
        for (Widget w : widgets) {
            w.update();
        }
    }

    public void breakpointOccured(SimTime breakpointTime) {
        realtimeField.setValue(0.0);
        updateControls();
    }

    private void log(Object... msg) {
        if (verbose) {
            Harness.log("SwingDisplay", msg);
        }
    }

    public static void main(String args[]) {
        Harness.initialize(SimTime.ZERO, false, false, false, false);
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new SwingDisplay(true).setVisible(true);
            }
        });
    }
}
