package simulator.payloads;

import simulator.framework.*;

/**
 * Framework state that for the coin return.  Pulsing true will cause a coin to be returned.
 */
public class CoinOutPayload extends PhysicalPayload {

    public boolean value;

    /**
     * <code>floorNumber</code> is between 1 and {@link
     * simulator.framework.Elevator#numFloors}; <code>hallway</code> is either
     * <code>Hallway.FRONT</code> or <code>Hallway.BACK</code>
     */
    public CoinOutPayload() {
        super(PhysicalPayload.CoinOutEvent);
        this.value = false;
        setName("CoinOutPayload");

    }

    public CoinOutPayload set(boolean buttonValue) {
        this.value = buttonValue;
        return this;
    }

    @Override
    public void copyFrom(Payload p) {
        super.copyFrom(p);
        CoinOutPayload c = (CoinOutPayload) p;
        value = c.value;
    }

    /**
     * Returns a String that represents the value of this object.
     * @return a string representation of the receiver
     */
    @Override
    public String toString() {
        return super.toString() + "(" + value + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        CoinOutPayload c = new CoinOutPayload();
        c.copyFrom(this);
        return c;
    }
}
