package simulator.payloads;

import simulator.framework.*;

/**
 * Framework message sent by a controller is a command to the light actuator
 * to turn the light on or off; framework message from the light indicates
 * whether the light is on or off.
 *  
 * @author jdevale
 */
public class LightPayload extends PhysicalPayload {

    final public int index;
    public boolean lighted;

    public LightPayload(int index) {
        super(PhysicalPayload.LightEvent, ReplicationComputer.computeReplicationId(index));
        this.index = index;
        lighted = false;

        setName("LightPayload[" + index + "]");
    }

    public LightPayload set(boolean value) {
        this.lighted = value;
        return this;
    }

    @Override
    public void copyFrom(Payload p) {
        super.copyFrom(p);
        LightPayload c = (LightPayload) p;
        if (this.index != c.index) {
            throw new RuntimeException("Wrong replication instance of " + this);
        }
        lighted = c.lighted;
    }

    @Override
    public String toString() {
        return super.toString() + "(" + lighted + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        LightPayload c = new LightPayload(index);
        c.copyFrom(this);
        return c;
    }
}
