package simulator.payloads;

import jSimPack.SimTime;
import simulator.framework.Harness;
import simulator.framework.Networkable;

/**
 * A message sent between components in the elevator.
 */
public abstract class Payload implements Comparable<Payload>, Cloneable {

    private int type;
    private SimTime timeStamp;
    private String name;

    public Payload(Payload p) {
        this.type = p.type;
        this.timeStamp = p.timeStamp;
        this.name = p.name;
    }

    public Payload(int type) {
        this.type = type;
    }

    public abstract void deliverTo(Networkable networkable);

    @Override
    public abstract Payload clone();

    public int getType() {
        return type;
    }

    @Override
    public final int hashCode() {
        return type;
    }

    @Override
    public final boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj instanceof Payload) {
            return ((Payload) obj).type == this.type;
        }
        return false;
    }

    public int compareTo(Payload mp) {
        if (this.type < mp.type) {
            return -1;
        }
        if (this.type > mp.type) {
            return 1;
        }
        return 0;
    }

    public final SimTime getTimeStamp() {
        return timeStamp;
    }

    public final void setTimeStamp(SimTime timeStamp) {
        this.timeStamp = timeStamp;
    }

    abstract public int getSize();

    public final String getName() {
        return name;
    }

    protected final void setName(String name) {
        this.name = name;
    }

    @Override
    public String toString() {
        return name;
    }

    /**
     * Copies the state of the given object into this object's state.
     * Subclasses that override this method should also call this
     * implementation.
     *
     * @param p the object whose state should be copied into this object
     */
    public void copyFrom(Payload p) {
        this.type = p.type;
        this.timeStamp = p.timeStamp;
        this.name = p.name;
    }
}
