package simulator.payloads;

import simulator.framework.*;

/**
 * Framework message commands the vend motor to move the VC left and right on
 * its track.  Set to (Stop) at initialization.
 *
 * @author jdevale
 */
public class SodaTrayPayload extends PhysicalPayload {

    public int sodaValue;

    public SodaTrayPayload() {
        super(PhysicalPayload.SodaTrayEvent);
        sodaValue = 0;
        setName("SodaTrayPayload");
    }

    public void set(int sodaValue) {
        this.sodaValue = sodaValue;
    }

    public int getSodaValue() {
        return sodaValue;
    }

    @Override
    public void copyFrom(Payload src) {
        super.copyFrom(src);
        sodaValue = ((SodaTrayPayload) src).sodaValue;
    }

    @Override
    public String toString() {
        return super.toString() + "(" + sodaValue + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        SodaTrayPayload c = new SodaTrayPayload();
        c.copyFrom(this);
        return c;
    }
}
