package simulator.payloads;

import simulator.elevatormodules.VendMotor;
import simulator.framework.Networkable;

/**
 * Internal framework message that indicates the exact position of the Car in
 * the hoistway.
 *
 * @author jdevale
 */
public class VendCarriagePositionPayload extends PhysicalPayload {

    /**
     * Location of the VC in mm.  0 is in front of chute 1.  700 units to the right, at
     * position 700 is the front of chute 8.  */
    public double position;

    public VendCarriagePositionPayload() {
        super(PhysicalPayload.VendCarriagePositionEvent);
        position = 0;
        setName("CarPositionPayload");
    }

    public void set(int value) {
        if (value < VendMotor.MIN_POSITION || value > VendMotor.MAX_POSITION) {
            throw new RuntimeException("Attempted to set out-of-range position value " + value);
        }
        this.position = value;
    }

    @Override
    public void copyFrom(Payload src) {
        super.copyFrom(src);
        position = ((VendCarriagePositionPayload) src).position;
    }

    @Override
    public String toString() {
        return super.toString() + "(" + position + ")";
    }

    @Override
    public void deliverTo(Networkable networkable) {
        networkable.receive(this);
    }

    @Override
    public Payload clone() {
        VendCarriagePositionPayload c = new VendCarriagePositionPayload();
        c.copyFrom(this);
        return c;
    }
}
